package edu.mit.jwi.data;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class BufferedReaderUtf8{
	
	public static final String FILENAME = "filename";
	public static final String INPUT_STREAM = "input_stream";
	public static final String TEXT_STRING = "text_string"; // The input already is the text content
	
	private BufferedReader br = null;
	private ByteArrayInputStream bais = null;
	private FileInputStream fis = null;
	private InputStreamReader isr = null;
	private String inputType = "";
	
	public BufferedReaderUtf8(InputStream inputStream, String type)
			throws IOException{
		//--------------------------------------
		// Type == INPUT_STREAM
		if (type.equals(INPUT_STREAM)){
			isr = new InputStreamReader(inputStream, "UTF8");
			br = new BufferedReader(isr);
			inputType = INPUT_STREAM;
		}
	}			
	
	/**
	 * @param inputString When type==FILENAME, this param is the filename. When
	 * type==TEXT_STRING, this param is the content of the stream.
	 * @param type
	 * @throws IOException
	 */
	public BufferedReaderUtf8(String inputString, String type)
		throws IOException{
		//--------------------------------------
		// Depending on type, instantiate differently
		
		//--------------------------------------
		// Type == FILENAME
		// Param inputString is treated as filename
		if (type.equals(FILENAME)){
			fis = new FileInputStream(inputString);
			isr = new InputStreamReader(fis, "UTF8");
			br = new BufferedReader(isr);
			inputType = FILENAME;
		}
		
		//--------------------------------------
		// Type == TEXT_STRING
		// Param inputString is treated as content of the stream
		else if (type.equals(TEXT_STRING)){
			bais = new ByteArrayInputStream(inputString.getBytes("UTF8"));
			isr = new InputStreamReader(bais, "UTF8");
			br = new BufferedReader(isr);
			inputType = TEXT_STRING;
		}
	}
	
	public void close() throws IOException{
		br.close();
		isr.close();
		if (inputType.equals(FILENAME)){
			fis.close();
		}
		else if (inputType.equals(INPUT_STREAM)){
			// Nothing to do
		}
		else if (inputType.equals(TEXT_STRING)){
			bais.close();
		}
	}
	
	public String readLine() throws IOException{
		return br.readLine();
	}
}
