/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.bebe.bean;

import java.util.ArrayList;
import java.util.List;

public class CanonicalEntity extends Entity{
	
	protected String dictionary = "";
	protected String id = ""; // For UMLS entities, this would be CUI
	protected List<String> semanticTypes = null; // For UMLS entities, for now only used by hadoop strings in Sentence
	
	public CanonicalEntity(String inputDictionary, String inputId, String inputName, String inputSelectionCriterion){
		this(inputDictionary, inputId, inputName, 0, inputSelectionCriterion, false);
	}
	
	public CanonicalEntity(String inputDictionary, String inputId, String inputName, double inputScore,
			String inputSelectionCriterion){
		this(inputDictionary, inputId, inputName, inputScore, inputSelectionCriterion, false);
	}
	
	public CanonicalEntity(String inputDictionary, String inputId, String inputName, int inputScore,
			String inputSelectionCriterion){
		this(inputDictionary, inputId, inputName, (double)inputScore, inputSelectionCriterion, false);
	}
	
	public CanonicalEntity(String inputDictionary, String inputId, String inputName, double inputScore,
			String inputSelectionCriterion, boolean inputAnchorFlag){
		super(inputName, inputScore, inputAnchorFlag, inputSelectionCriterion);
		dictionary = inputDictionary;
		id = inputId;
	}
	
	public void addSemanticType(String inputSemanticType){
		if (semanticTypes == null){
			semanticTypes = new ArrayList<String>();
		}
		semanticTypes.add(inputSemanticType);
	}
	
	public CanonicalEntity clone(){
		return new CanonicalEntity(dictionary, id, name, score, selectionCriterion, anchorFlag);
	}
	
	public String getDictionaryName(){
		return dictionary;
	}
	
	public String getId(){
		return id;
	}
	
	public List<String> getSemanticTypes(){
		return semanticTypes;
	}
	
	public void setSemanticTypes(List<String> inputSemanticTypes){
		semanticTypes = inputSemanticTypes;
	}
	
	public List<String> toArchiveStrings(){
		List<String> returnList = new ArrayList<String>();
		String DELIMITER = "~@~";
		
		returnList.add(getScore() + DELIMITER + getType() + DELIMITER + getName() + DELIMITER + getSelectionCriterion()
				+ DELIMITER + getDictionaryName() + DELIMITER + getId());
		
		return returnList;
	}
	
	public String toDebugString(){
		return toString();
	}
	
	public String toString(){
		String returnString = "=== Canonical entity ===\n";
		returnString += "Dictionary:          " + dictionary + "\n";
		returnString += "ID:                  " + id + "\n";
		returnString += "Name:                " + name + "\n";
		returnString += "Score:               " + score + "\n";
		returnString += "Selection criterion: " + selectionCriterion + "\n";
		returnString += "Is anchor:           " + anchorFlag + "\n";
		
		return returnString;
	}
}