/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.bebe.bean;

import java.util.ArrayList;
import java.util.List;

import mpii.biotm.util.EmptyStringUtf8;

public class Entity implements Comparable<Entity>{
	
	protected boolean anchorFlag = false; // TODO: remove?
	protected String name = "";
	protected double score = 0;
	protected String selectionCriterion = "";
	
	public Entity(String inputName){
		this(inputName, 0, false, "");
	}
	
	public Entity(String inputName, boolean inputAnchorFlag){
		this(inputName, 0, inputAnchorFlag, "");
	}
	
	public Entity(String inputName, double inputScore){
		this(inputName, inputScore, false, "");
	}
	
	public Entity(String inputName, String inputSelectionCriterion){
		this(inputName, 0, false, inputSelectionCriterion);
	}
	
	public Entity(String inputName, double inputScore, String inputSelectionCriterion){
		this(inputName, inputScore, false, inputSelectionCriterion);
	}
	
	public Entity(String inputName, double inputScore, boolean inputAnchorFlag, String inputSelectionCriterion){
		anchorFlag = inputAnchorFlag;
		name = EmptyStringUtf8.getEmptyString() + inputName; // Force the string to be UTF-8
		score = inputScore;
		selectionCriterion = inputSelectionCriterion;
	}
	
	public int compareTo(Entity anotherEntity){
		int returnInt = 0;
		
		// Compare only by the score, in descending order,
		// i.e. bigger score comes "earlier"
		
		double anotherScore = anotherEntity.getScore();
		
		if (score < anotherScore){
			returnInt = 1;
		}
		else if (score > anotherScore){
			returnInt = -1;
		}
		// else score == anotherScore, nothing to do, as returnInt is already 0
		
		return returnInt;
	}
	
	public String getName(){
		return name;
	}
	
	public double getScore(){
		return score;
	}
	
	public String getSelectionCriterion(){
		return selectionCriterion;
	}
	
	/** The name of the (sub) class will tell what kind of entity the object is,
	 * i.e. "plain", canonical, etc. */
	public String getType(){
		return this.getClass().getSimpleName();
	}
	
	public boolean isAnchor(){
		return anchorFlag;
	}
	
	public void setScore(double inputScore){
		score = inputScore;
	}
	
	public List<String> toArchiveStrings(){
		List<String> returnList = new ArrayList<String>();
		String DELIMITER = "~@~";
		
		returnList.add(score + DELIMITER + getType() + DELIMITER + name + DELIMITER + selectionCriterion);
		
		return returnList;
	}
	
	public String toDebugString(){
		return toString();
	}
	
	public String toString(){
		String returnString = "=== Entity ===\n";
		returnString += "Name:                " + name + "\n";
		returnString += "Score:               " + score + "\n";
		returnString += "Is anchor:           " + anchorFlag + "\n";
		returnString += "Selection criterion: " + selectionCriterion + "\n";
		
		return returnString;
	}
}