/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.apache.log4j.Logger;

public class BufferedReaderUtf8{
	
	protected static final Logger logger = Logger.getLogger(BufferedReaderUtf8.class);
	
	public static final String FILENAME = "filename";
	public static final String INPUT_STREAM = "input_stream";
	public static final String TEXT_STRING = "text_string"; // The input already is the text content
	
	private BufferedReader br = null;
	private ByteArrayInputStream bais = null;
	private FileInputStream fis = null;
	private InputStreamReader isr = null;
	private String inputType = "";
	
	public BufferedReaderUtf8(InputStream inputStream, String type)
			throws IOException{
		//--------------------------------------
		// Type == INPUT_STREAM
		if (type.equals(INPUT_STREAM)){
			isr = new InputStreamReader(inputStream, "UTF8");
			br = new BufferedReader(isr);
			inputType = INPUT_STREAM;
		}
	}			
	
	/**
	 * @param inputString When type==FILENAME, this param is the filename. When
	 * type==TEXT_STRING, this param is the content of the stream.
	 * @param type
	 * @throws IOException
	 */
	public BufferedReaderUtf8(String inputString, String type)
		throws IOException{
		//--------------------------------------
		// Depending on type, instantiate differently
		
		//--------------------------------------
		// Type == FILENAME
		// Param inputString is treated as filename
		if (type.equals(FILENAME)){
			fis = new FileInputStream(inputString);
			isr = new InputStreamReader(fis, "UTF8");
			br = new BufferedReader(isr);
			inputType = FILENAME;
		}
		
		//--------------------------------------
		// Type == TEXT_STRING
		// Param inputString is treated as content of the stream
		else if (type.equals(TEXT_STRING)){
			bais = new ByteArrayInputStream(inputString.getBytes("UTF8"));
			isr = new InputStreamReader(bais, "UTF8");
			br = new BufferedReader(isr);
			inputType = TEXT_STRING;
		}
	}
	
	public void close() throws IOException{
		br.close();
		isr.close();
		if (inputType.equals(FILENAME)){
			fis.close();
		}
		else if (inputType.equals(INPUT_STREAM)){
			// Nothing to do
		}
		else if (inputType.equals(TEXT_STRING)){
			bais.close();
		}
	}
	
	public String readLine() throws IOException{
		return br.readLine();
	}
}
