/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.List;

import org.apache.log4j.Logger;

public class BufferedWriterUtf8{
	
	protected static final Logger logger = Logger.getLogger(BufferedWriterUtf8.class);
	
	public static final String FILENAME = "filename";
	public static final String TEXT_STRING = "text_string";
	
	public static final String APPEND = "append";
	public static final String OVERWRITE = "overwrite";
	
	private BufferedWriter bw = null;
	private ByteArrayOutputStream baos = null;
	private FileOutputStream fos = null;
	private OutputStreamWriter osw = null;
	private String outputType = "";
	
	/**
	 * This constructor is only suitable for file buffer. Default append mode is overwrite.
	 * @param filename
	 * @param inputOutputType
	 * @throws IOException
	 */
	public BufferedWriterUtf8(String filename, String type) throws IOException{
		this(filename, type, OVERWRITE);
	}
	
	/**
	 * This constructor is only suitable for file buffer. Choose between append and overwrite modes.
	 * @param filename
	 * @param inputOutputType
	 * @param inputAppendType
	 * @throws IOException
	 */
	public BufferedWriterUtf8(String filename, String type, String inputAppendType) throws IOException{
		if (type.equals(FILENAME)){
			if (inputAppendType.equals(APPEND)){
				fos = new FileOutputStream(filename, true);
			}
			else{
				fos = new FileOutputStream(filename, false);
			}
			osw = new OutputStreamWriter(fos, "UTF8");
			bw = new BufferedWriter(osw);
			outputType = FILENAME;
		}
		
		//-----------------------------------------
		// Error: wrong output type
		else{
			logger.error("Mismatch of using a filename with string stream output type");
		}
	}
	
	/**
	 * This constructor is only suitable for string buffer. Append mode is not available.
	 * @param type
	 * @throws UnsupportedEncodingException
	 */
	public BufferedWriterUtf8(String type) throws UnsupportedEncodingException{
		if (type.equals(TEXT_STRING)){
			baos = new ByteArrayOutputStream();
			osw = new OutputStreamWriter(baos, "UTF8");
			bw = new BufferedWriter(osw);
			outputType = TEXT_STRING;
		}
		
		//-----------------------------------------
		// Error: wrong output type
		else{
			logger.error("File output type indicated without a filename; use a different constructor");
		}
	}
	
	public void close() throws IOException{
		bw.close();
		osw.close();
		if (outputType.equals(FILENAME)){
			fos.close();
		}
		else if (outputType.equals(TEXT_STRING)){
			baos.close();
		}
	}
	
	public void flush() throws IOException{
		bw.flush();
	}
	
	public void write(String inputString) throws IOException{
		bw.write(inputString);
	}
	
	public void writeLine() throws IOException{
		bw.write("\n");
	}
	
	public void writeLine(String inputString) throws IOException{
		bw.write(inputString + "\n");
	}
	
	public void writeLines(List<String> inputStrings) throws IOException{
		for (String inputString : inputStrings){
			writeLine(inputString);
		}
	}
}
