/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.util.Comparator;

import edu.stanford.nlp.util.StringUtils;

public class ComparatorFilenamesByLongNumberSuffix implements Comparator<String>{

	public int compare(String s1, String s2){
		//--------------------------------------
		// Assumption: Both s1 and s2 have a file extension,
		// and just prior to the file extension there are integer values
		// E.g. /home/user/blah/ABC11.txt
		//      /home/user/blah/ABC100.txt
		// We want ABC11 to be sorted in front of ABC100.
		//
		// Actually, this method words for strings without file
		// extensions, as well as strings without alpha prefix.
		// E.g. 11 vs. 100
		//      will be sorted to 11 < 100
		
		//--------------------------------------
		// Extract the integer part of the input string
		Long long1 = extractLongPartFrom(s1);
		Long long2 = extractLongPartFrom(s2);
		
		return long1.compareTo(long2);
	}
	
	private Long extractLongPartFrom(String s){
		//--------------------------------------
		// Remove any path information from the string
		int slashPos = s.lastIndexOf("/");
		if (slashPos >= 0){
			s = s.substring(slashPos + 1);
		}

		//--------------------------------------
		// Remove any file extension from the string
		int dotPos = s.lastIndexOf(".");
		if (dotPos >= 0){
			s = s.substring(0, dotPos);
		}
		
		//--------------------------------------
		// Remove any prefix from the string before the
		// final occurrence of numbers
		int numberPosition = s.length() - 1;
		while (StringUtils.isNumeric(s.substring(numberPosition, numberPosition + 1)) && numberPosition >= 0){
			--numberPosition;
		}
		s = s.substring(numberPosition + 1);
		
		//--------------------------------------
		// Transform the remaining string into integer
		return Long.valueOf(s);
	}
}