/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.util.Comparator;
import java.util.Map;

public class CompareMapByValueIntegerDescending implements Comparator<Object>{
	// Compares both keys and values:
	// First order by values in descending order
	// Then order by keys in ascending order
	
	// Sample code to use this comparator:
	// CompareMapByValueIntegerDescending comparator =
	//		new CompareMapByValueIntegerDescending(unsorted);
	// TreeMap<String, Integer> sorted = new TreeMap<String, Integer>(comparator);
	// sorted.putAll(unsorted);
	// for (String key : sorted.keySet()){
	//     logger.debug(key + " :: " + sorted.get(key));
	// }
	
	Map<?, Integer> map;
	public CompareMapByValueIntegerDescending(Map<?, Integer> map){
		this.map = map;
	}

	public int compare(Object a, Object b){
		int returnInt = 0;
		
		int stringComparison = a.toString().compareTo(b.toString());
		Integer intA = (Integer)map.get(a);
		Integer intB = (Integer)map.get(b);
		int intComparison = intA.compareTo(intB);
		
		if (intComparison == 0){
			returnInt = stringComparison;
		}
		else{
			returnInt = -intComparison;
		}
		
		return returnInt;
	}
}