/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

public class DetermineIdPrefix{
	
	public static String byMillion(int id){
		return byMillion((long) id);
	}
	
	public static String byMillion(long id){
		// Divide the ID by 1,000,000 and then return the integer quotient
		// E.g. PMCID 1234568 ==> 1
		int quotient = 0;
		if (id >= 1000000){
			quotient = (int) Math.floor(id/1000000);
		}
		return String.valueOf(quotient);
	}
	
	public static String byMillion(String id){
		return byMillion(Long.parseLong(id));
	}
	
	public static String byMillionPadded(int id){
		return byMillionPadded((long) id);
	}
	
	public static String byMillionPadded(long id){
		return padToThreeChars(byMillion(id));
	}
	
	public static String byMillionPadded(String id){
		return padToThreeChars(byMillion(Long.parseLong(id)));
	}
	
	public static String byThousand(int id){
		return byThousand((long) id);
	}
	
	public static String byThousand(long id){
		// Divide the ID by 1,000 and get integer quotient A.
		// Further divide A by 1,000 and get the integer remainder B.
		// Return B.
		int quotient = 0;
		int remainder = 0;
		if (id >= 1000){
			quotient = (int) Math.floor(id/1000);
			remainder = (int) Math.floor(quotient%1000);
		}
		return String.valueOf(remainder);
	}
	
	public static String byThousand(String id){
		return byThousand(Long.parseLong(id));
	}
	
	public static String byThousandPadded(int id){
		return byThousandPadded((long) id);
	}
	
	public static String byThousandPadded(long id){
		return padToThreeChars(byThousand(id));
	}
	
	public static String byThousandPadded(String id){
		return padToThreeChars(byThousand(Long.parseLong(id)));
	}
	
	public static String padToThreeChars(String inputString){
		// Prefix the input string with '0's until the
		// string is 3 characters long.
		while (inputString.length() < 3){
			inputString = "0" + inputString;
		}
		return inputString;
	}
}
