/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.util.List;

public class InsertTabs{
	
	private static String prefix = "";
	
	/**
	 * Given a string, insert a given number of tabs:
	 * - at the beginning of the string;
	 * - after every newline, except the final newline.
	 */
	public static String insertTabs(String inputString, int numberOfTabs){
		
		//-----------------------------------------
		// Reset the prefix if necessary
		if (prefix.length() != numberOfTabs){
			prefix = "";
			for (int i=0; i < numberOfTabs; ++i){
				prefix += "\t";
			}
		}
		
		//-----------------------------------------
		// Place the prefix at the beginning of the string
		inputString = prefix + inputString;
		
		//-----------------------------------------
		// Place a prefix after every newline
		inputString = inputString.replaceAll("\n", "\n" + prefix);
		
		//-----------------------------------------
		// See if there is a final newline. If yes, remove the
		// unwanted prefix there.
		if (inputString.endsWith(prefix)){
			inputString = inputString.substring(0, inputString.length() - numberOfTabs);
		}
		
		return inputString;
	}
	
	/**
	 * Given a list of strings, insert tabs for each member.
	 */
	public static List<String> insertTabs(List<String> inputStrings, int numberOfTabs){
		for (int i=0; i < inputStrings.size(); ++i){
			inputStrings.set(i, insertTabs(inputStrings.get(i), numberOfTabs));
		}
		return inputStrings;
	}
}
