/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import org.apache.commons.lang3.CharUtils;

public class ProcessNonAlphanumericCharacter{
	
	/**
	 * Remove all non-alphanumeric characters. Input is assumed to be UTF-8.
	 */
	public static String remove(String inputString){
		return replaceWith(inputString, "");
	}
	
	/**
	 * Replace all non-alphanumeric characters with the given replacement (empty
	 * string is also allowed). Input is assumed to be UTF-8.
	 */
	public static String replaceWith(String inputString, String replacement){
		String returnString = "";
		
		for (int i = 0; i < inputString.length(); ++i){
			char c = inputString.charAt(i);
			if (CharUtils.isAsciiAlphanumeric(c)){
				returnString += c;
			}
			else{
				returnString += replacement;
			}
		}
		
		return returnString;
	}
	
	/**
	 * Replace all non-alphanumeric characters with spaces. Input is assumed to
	 * be UTF-8.
	 */
	public static String replaceWithSpace(String inputString){
		// The source file is saved in UTF-8. Since I am paranoid, I'll just
		// make sure it's the ASCII space I'm using here.
		char c = (char)32;
		return replaceWith(inputString, c + ""); 
	}
	
	public static String trimLeadingTrailing(String inputString){
		//-----------------------------------------
		// Trim leading and trailing non-alphanumeric chars
		while (!inputString.isEmpty() &&
				CharUtils.isAscii(inputString.charAt(0)) &&
				!CharUtils.isAsciiAlphanumeric(inputString.charAt(0))){
			inputString = inputString.substring(1);
		}

		while (!inputString.isEmpty() &&
				CharUtils.isAscii(inputString.charAt(inputString.length() - 1)) &&
				!CharUtils.isAsciiAlphanumeric(inputString.charAt(inputString.length() - 1))){
			inputString = inputString.substring(0, inputString.length() - 1);
		}
		return inputString;
	}
}
