/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.apache.commons.lang3.StringUtils;

public class ProcessStopWord{
	
	private static final Set<String> stopWords = new HashSet<String>();
	static{
		stopWords.add("a");
		stopWords.add("about");
		stopWords.add("above");
		stopWords.add("across");
		stopWords.add("after");
		stopWords.add("again");
		stopWords.add("all");
		stopWords.add("almost");
		stopWords.add("along");
		stopWords.add("already");
		stopWords.add("also");
		stopWords.add("although");
		stopWords.add("always");
		stopWords.add("am");
		stopWords.add("among");
		stopWords.add("amongst");
		stopWords.add("an");
		stopWords.add("and");
		stopWords.add("another");
		stopWords.add("any");
		stopWords.add("anybody");
		stopWords.add("anyhow");
		stopWords.add("anyone");
		stopWords.add("anyway");
		stopWords.add("anyways");
		stopWords.add("anywhere");
		stopWords.add("apart");
		stopWords.add("are");
		stopWords.add("aren't");
		stopWords.add("around");
		stopWords.add("as");
		stopWords.add("aside");
		stopWords.add("at");
		stopWords.add("be");
		stopWords.add("been");
		stopWords.add("being");
		stopWords.add("before");
		stopWords.add("beforehand");
		stopWords.add("behind");
		stopWords.add("below");
		stopWords.add("beside");
		stopWords.add("besides");
		stopWords.add("between");
		stopWords.add("beyond");
		stopWords.add("but");
		stopWords.add("by");
		stopWords.add("can");
		stopWords.add("can't");
		stopWords.add("cannot");
		stopWords.add("could");
		stopWords.add("couldn't");
		stopWords.add("despite");
		stopWords.add("did");
		stopWords.add("didn't");
		stopWords.add("do");
		stopWords.add("does");
		stopWords.add("doesn't");
		stopWords.add("don't");
		stopWords.add("doing");
		stopWords.add("done");
		stopWords.add("downwards");
		stopWords.add("during");
		stopWords.add("either");
		stopWords.add("else");
		stopWords.add("elsewhere");
		stopWords.add("everybody");
		stopWords.add("everyone");
		stopWords.add("everything");
		stopWords.add("everywhere");
		stopWords.add("for");
		stopWords.add("from");
		stopWords.add("further");
		stopWords.add("furthermore");
		stopWords.add("had");
		stopWords.add("hadn't");
		stopWords.add("has");
		stopWords.add("hasn't");
		stopWords.add("have");
		stopWords.add("haven't");
		stopWords.add("having");
		stopWords.add("he");
		stopWords.add("her");
		stopWords.add("here");
		stopWords.add("hereafter");
		stopWords.add("hereby");
		stopWords.add("herein");
		stopWords.add("hereupon");
		stopWords.add("him");
		stopWords.add("his");
		stopWords.add("hither");
		stopWords.add("however");
		stopWords.add("I"); // Yes, this is not ideal... we consider the roman numeral "I" as a stop word now
		stopWords.add("if");
		stopWords.add("in");
		stopWords.add("instead");
		stopWords.add("into");
		stopWords.add("inward");
		stopWords.add("is");
		stopWords.add("isn't");
		stopWords.add("it");
		stopWords.add("let");
		stopWords.add("like");
		stopWords.add("may");
		stopWords.add("me");
		stopWords.add("meanwhile");
		stopWords.add("more");
		stopWords.add("much");
		stopWords.add("my");
		stopWords.add("near");
		stopWords.add("neither");
		stopWords.add("nevertheless");
		stopWords.add("no");
		stopWords.add("nobody");
		stopWords.add("nor");
		stopWords.add("not");
		stopWords.add("nothing");
		stopWords.add("nowhere");
		stopWords.add("n't");
		stopWords.add("of");
		stopWords.add("off");
		stopWords.add("often");
		stopWords.add("on");
		stopWords.add("only");
		stopWords.add("onto");
		stopWords.add("or");
		stopWords.add("other");
		stopWords.add("otherwise");
		stopWords.add("our");
		stopWords.add("she");
		stopWords.add("should");
		stopWords.add("shouldn't");
		stopWords.add("some");
		stopWords.add("somebody");
		stopWords.add("somehow");
		stopWords.add("someone");
		stopWords.add("something");
		stopWords.add("sometime");
		stopWords.add("sometimes");
		stopWords.add("somewhat");
		stopWords.add("somewhere");
		stopWords.add("still");
		stopWords.add("such");
		stopWords.add("than");
		stopWords.add("that");
		stopWords.add("the");
		stopWords.add("their");
		stopWords.add("them");
		stopWords.add("there");
		stopWords.add("thereafter");
		stopWords.add("thereby");
		stopWords.add("therefore");
		stopWords.add("therein");
		stopWords.add("theres");
		stopWords.add("thereupon");
		stopWords.add("these");
		stopWords.add("they");
		stopWords.add("this");
		stopWords.add("those");
		stopWords.add("though");
		stopWords.add("through");
		stopWords.add("throughout");
		stopWords.add("to");
		stopWords.add("toward");
		stopWords.add("towards");
		stopWords.add("under");
		stopWords.add("unless");
		stopWords.add("until");
		stopWords.add("unto");
		stopWords.add("upon");
		stopWords.add("very");
		stopWords.add("was");
		stopWords.add("wasn't");
		stopWords.add("we");
		stopWords.add("were");
		stopWords.add("what");
		stopWords.add("when");
		stopWords.add("whence");
		stopWords.add("where");
		stopWords.add("whereafter");
		stopWords.add("whereas");
		stopWords.add("whereby");
		stopWords.add("wherein");
		stopWords.add("whereupon");
		stopWords.add("wherever");
		stopWords.add("whether");
		stopWords.add("which");
		stopWords.add("while");
		stopWords.add("whither");
		stopWords.add("who");
		stopWords.add("whoever");
		stopWords.add("whom");
		stopWords.add("whose");
		stopWords.add("with");
		stopWords.add("within");
		stopWords.add("without");
		stopWords.add("won't");
		stopWords.add("would");
		stopWords.add("wouldn't");
		stopWords.add("you");
		stopWords.add("your");
		stopWords.add("'s");
	}

	public static boolean isStopWord(String inputWord){
		boolean returnBool = false;
		
		// Except for "I", change the input word to lower case
		if (!inputWord.equals("I")){
			inputWord = inputWord.toLowerCase();
		}
		
		if (stopWords.contains(inputWord)){
			returnBool = true;
		}
		return returnBool;
	}
	
	public static String removeStopWords(String inputWords){
		// Given a string of words, remove the stop words, and
		// return the streamlined string
		//
		// The input should contain original upper / lower cases, because
		// we want to be able to tell "I" is a stop word but "i" is not
		//
		// Assumption: input contains no punctuations
		List<String> words = Arrays.asList(inputWords.split(" "));
		words = removeStopWords(words);
		return StringUtils.join(words, " ");
	}
	
	public static List<String> removeStopWords(List<String> inputWords){
		// Give a list of words, remove elements in the list that
		// are stop words, and return the streamlined list
		//
		// The input should contain original upper / lower cases, because
		// we want to be able to tell "I" is a stop word but "i" is not
		//
		// Note: I'm not using List.remove() because for whatever reason
		//       that method is not supported on the input list, and I get
		//       UnsupportedOperationException
		List<String> returnList = new ArrayList<String>();
		for (String inputWord : inputWords){
			if (!isStopWord(inputWord)){
				returnList.add(inputWord);
			}
		}
		
		return returnList;
	}
}