/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

public class ProcessUpperLowerCase{
	
	public static boolean allLettersAsUpperCase(String string){
		// Disregard non-letter characters. If all the letters are
		// upper case, return true. If the input string does not
		// have any letters, the outcome is undefined (but arbitrarily
		// returning false).
		boolean returnBool = false;
		
		String tempString = string.replaceAll("[^a-zA-Z]", "");
		if (StringUtils.isAllUpperCase(tempString)){
			returnBool = true;
		}
		
		return returnBool;
	}
	
	public static boolean moreThanCamelCase(String string){
		// Disregard non-letter characters. If there is one word
		// within the given string that:
		// - Either has a mixture of upper and lower cases, and it's not
		// "just" camel case (initial upper case followed by all lower cases),
		//   OR
		// - The word is all upper case
		// then return true.
		boolean returnBool = false;
		
		String tempString = string.replaceAll("[^a-zA-Z\\s]", "");
		List<String> words = Arrays.asList(tempString.split("\\W"));
		int i = 0;
		while (i < words.size() && !returnBool){
			String word = words.get(i);
			if (allLettersAsUpperCase(word)){
				returnBool = true;
			}
			else{
				int numberOfUpperCaseLetters = 0;
				int numberOfLowerCaseLetters = 0;
				boolean initialLetterIsUpperCase = false;

				for (int j = 0; j < word.length(); ++j){
					String oneChar = word.substring(j, j + 1);
					boolean isUpperCase = StringUtils.isAllUpperCase(oneChar);
					if (isUpperCase){
						++numberOfUpperCaseLetters;
						if (j == 0){
							initialLetterIsUpperCase = true;
						}
					}
					else{
						++numberOfLowerCaseLetters;
					}
				}

				if (numberOfUpperCaseLetters > 1 && numberOfLowerCaseLetters > 0){ // E.g. IgG
					returnBool = true;
				}
				else if (numberOfUpperCaseLetters == 1 && numberOfLowerCaseLetters > 0 && !initialLetterIsUpperCase){ // E.g. aKb
					returnBool = true;
				}
			}
			
			++i;
		}
		
		return returnBool;
	}
}