/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.util;

import java.util.Date;
import java.util.Random;

public class RandomNumber{
	
	private static Random generator = null;
	private static boolean initialized = false;
	
	public static double getRandomNumberBetweenZeroAndOne(){
		return generator.nextDouble();
	}

	public static int getRandomNumberInRange(int min, int max){
		return (int) getRandomNumberInRange((long) min, (long) max);
	}
	
	public static long getRandomNumberInRange(long min, long max){
		// Randomly generate an integer in between the supplied min and max values, inclusive
		long returnLong = min;
		
		// Only continue if min and max are valid inputs
		if (min < max){
			long difference = max - min;
			double randomNumber = generator.nextDouble();
			returnLong = (long) (min + Math.round(randomNumber * difference));
		}
		
		return returnLong;
	}
	
	/** Initialize with a seed derived from current time. */ 
	public static void initialize(){
		//-----------------------------------------
		// Derive a seed from current time
		Date date = new Date();
		initialize(date.getTime());
	}
	
	public static void initialize(long seed){
		// Note to self: The Random class guarantees that the same seed generates
		// the same sequence of numbers.
		if (!initialized){
			generator = new Random(seed);
			initialized = true;
		}
	}
	
	public static void main(String[] args) throws InterruptedException{
		//-----------------------------------------
		// Quick test
		initialize();
		for (int i = 0; i < 20; ++i){
			System.out.print(getRandomNumberInRange(10, 20) + "\t");
		}
	}
}
