/*
 * Copyright 2015 Amy Siu (siu@mpi-inf.mpg.de)
 *
 * This file is part of the BEBE experimental software package.
 *
 * BEBE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BEBE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BEBE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package mpii.biotm.wrapper.umls;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import mpii.biotm.util.ProcessUpperLowerCase;
import mpii.biotm.util.ReadFromFile;

import org.apache.log4j.Logger;

public class LookupMostCommonSenseCui{
	
	protected static final Logger logger = Logger.getLogger(LookupMostCommonSenseCui.class);
	
	private static boolean initialized = false;
	private static Map<String, Integer> conceptNamesToMostCommonSenseCuis = null;
	
	public static void initialize(List<String> inputLines){
		if (!initialized){
			logger.info("Reading in concept names and CUIs...");
			initializeLookupMap(inputLines);
			logger.info("Done");
			
			initialized = true;
		}
	}

	public static void initialize(Properties settings) throws Exception{
		if (!initialized){
			logger.info("Reading in concept names and CUIs...");
			
			//-----------------------------------------
			// Read in the file
			String filename = settings.getProperty("umls_most_common_sense_cuis_concept_names_filename");
			List<String> inputLines = ReadFromFile.readFileToListOfStrings(filename);
			initializeLookupMap(inputLines);

			logger.info("Done");
			
			initialized = true;
		}
	}
	
	private static void initializeLookupMap(List<String> inputLines){
		conceptNamesToMostCommonSenseCuis = new HashMap<String, Integer>();
		for (String inputLine : inputLines){
			int tabPosition = inputLine.indexOf("\t");
			String conceptName = inputLine.substring(0, tabPosition);
			String cui = inputLine.substring(tabPosition + 1);
			int cuiAsInt = CuiAndVersion.toInt(cui);
			conceptNamesToMostCommonSenseCuis.put(conceptName, cuiAsInt);
		}
	}
	
	public static String lookupReturnCui(String conceptName){
		//-----------------------------------------
		// Given a concept name, look up its most common sense CUI.
		// If there is none, return empty string.
		String cui = "";
		int cuiAsInt = lookupReturnCuiAsInt(conceptName);
		
		if (cuiAsInt != -1){
			cui = CuiAndVersion.toCui(cuiAsInt);
		}
		
		return cui;
	}
	
	public static int lookupReturnCuiAsInt(String conceptName){
		//-----------------------------------------
		// Given a concept name, look up its most common sense CUI as integer.
		// If there is none, return -1.
		int cuiAsInt = -1;
		
		if (!ProcessUpperLowerCase.moreThanCamelCase(conceptName)){
			conceptName = conceptName.toLowerCase();
		}
		
		if (conceptNamesToMostCommonSenseCuis.containsKey(conceptName)){
			cuiAsInt = conceptNamesToMostCommonSenseCuis.get(conceptName);
		}
		
		return cuiAsInt;
	}
}